package jdkl.PicJ.DSP;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import java.awt.image.BufferedImage;
import javax.imageio.ImageIO;

import jdkl.PicJ.Browser.ImageFilenameFilter;

public class ImageFrame extends JFrame implements ActionListener {

	private File currentFile;
	private File dirFile;
	private int fileIndex;
	private JFileChooser fcRead;
	private JFileChooser fcWrite;
	private LinkedList imageList;			//store BufferedImage(s)
	private int listIndex;					//current BufferedImage index
	private ImageProcess imageProcess;		//image process methods
	private ImageComponent imageComponent;	//container of BufferedImage

	//ImageFrame
	public ImageFrame(File f) {
		super();
		getContentPane().setLayout(new BorderLayout());

		//initialize list
		imageList = new LinkedList();
		currentFile = f;
		dirFile = f.getParentFile();
		fileIndex = 0;
		String filename = currentFile.getName();
		String[] tmpFileList = dirFile.list(new ImageFilenameFilter());
		for (int i = 0; i < tmpFileList.length; ++i) {
			if (tmpFileList[i].equals(filename)) {
				fileIndex = i;
				break;
			}
		}
		//read image from file
		Image image = Toolkit.getDefaultToolkit().getImage(f.toString());
		imageComponent = new ImageComponent(image);
		//add ImageComponent to list
		imageList.add(imageComponent.getBufferedImage());
		listIndex = 0;
		//set width, height
		Dimension d = imageComponent.getPreferredSize();
		int dx = (int)d.getWidth();
		int dy = (int)d.getHeight();
		//set title
		setTitle(currentFile.getName() + " " + dx + "X" + dy);
		//add imagecomponent
		getContentPane().add(new JScrollPane(imageComponent));
		//menu bar
		JMenuBar menuBar = createMenuBar();
		setJMenuBar(menuBar);
		//initial imageProcess
		imageProcess = new ImageProcess();
		//W@ U@
		addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent ke) {
				if (ke.getKeyCode() == KeyEvent.VK_PAGE_UP) {
					pageUp();
				} else if (ke.getKeyCode() == KeyEvent.VK_PAGE_DOWN) {
					pageDown();
				}
			}
		});

		//]w
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		//]̤jùjp
		Dimension sd = Toolkit.getDefaultToolkit().getScreenSize();
		int maxdx = (int)sd.getWidth();
		int maxdy = (int)sd.getHeight();
		//dx = dx + 11;
		//dy = dy + 55;
		pack();
		dx = this.getWidth();
		dy = this.getHeight();
		if (dx > maxdx) {
			dx = maxdx;
		}
		if (dy > maxdy) {
			dy = maxdy;
		}
		setSize(dx, dy);
		setLocation((maxdx - dx) / 2, (maxdy - dy) / 2);
		setVisible(true);
	}

	private JMenuBar createMenuBar() {
		JMenuBar menubar = new JMenuBar();

		//file menu
		JMenu fileMenu = new JMenu("ɮ");
		fileMenu.add(makeMenuItem("}..."));
		fileMenu.addSeparator();
		fileMenu.add(makeMenuItem("s"));
		fileMenu.add(makeMenuItem("sɬ..."));
		fileMenu.addSeparator();
		fileMenu.add(makeMenuItem("}"));
		menubar.add(fileMenu);
		//edit menu
		JMenu editMenu = new JMenu("s");
		editMenu.add(makeMenuItem("_"));
		editMenu.add(makeMenuItem(""));
		menubar.add(editMenu);
		//view menu
		JMenu viewMenu = new JMenu("˵");
		JMenu view_zoomMenu = new JMenu("Y");
		view_zoomMenu.add(makeMenuItem("̤"));
		view_zoomMenu.add(makeMenuItem("̤jp"));
		viewMenu.add(view_zoomMenu);
		viewMenu.add(makeMenuItem("Ϭ"));
		viewMenu.add(makeMenuItem("m"));
		viewMenu.add(makeMenuItem("ⶥ"));
		viewMenu.add(makeMenuItem("z"));
		viewMenu.add(makeMenuItem(""));
		viewMenu.addSeparator();
		JMenu view_mirrorMenu = new JMenu("g");
		view_mirrorMenu.add(makeMenuItem("g"));
		view_mirrorMenu.add(makeMenuItem("g"));
		viewMenu.add(view_mirrorMenu);
		JMenu view_turnMenu = new JMenu("");
		view_turnMenu.add(makeMenuItem("180"));
		view_turnMenu.add(makeMenuItem("k90"));
		view_turnMenu.add(makeMenuItem("90"));
		view_turnMenu.add(makeMenuItem("N"));
		viewMenu.add(view_turnMenu);
		menubar.add(viewMenu);
		//filter menu
		JMenu filterMenu = new JMenu("o");
		filterMenu.add(makeMenuItem("ɧJ"));
		filterMenu.add(makeMenuItem("W[I"));
		filterMenu.add(makeMenuItem("ҽk"));
		filterMenu.add(makeMenuItem("hI"));
		filterMenu.addSeparator();
		filterMenu.add(makeMenuItem("BJ"));
		filterMenu.add(makeMenuItem(""));
		menubar.add(filterMenu);
		//Help
		JMenu helpMenu = new JMenu("");
		helpMenu.add(makeMenuItem(""));
		menubar.add(helpMenu);
		return menubar;
	}

	private JMenuItem makeMenuItem(String label) {
		JMenuItem item = new JMenuItem(label);
		item.addActionListener(this);
		return item;
	}

	public void actionPerformed(ActionEvent ae) {
		String command = ae.getActionCommand();
		BufferedImage currentBufferedImage = (BufferedImage)imageList.get(listIndex);
		BufferedImage tmp = null;
		if (command.equals("}...")) {
			openFile();
			return;
		} else if (command.equals("s")) {
			saveFile();
			return;
		} else if (command.equals("sɬ...")) {
			saveAsFile();
			return;
		} else if (command.equals("}")) {
			dispose();
			return;
		} else if (command.equals("_")) {
			undo();
			return;
		} else if (command.equals("")) {
			redo();
			return;
		} else if (command.equals("̤")) {
			String result = (String)JOptionPane.showInputDialog(this,
					"пJY񪺤(%)G",
					"Y", JOptionPane.OK_CANCEL_OPTION, null, null, null);
			if (result == null) {
				return;
			}
			float val = Float.parseFloat(result);
			ImageComponent ictemp = new ImageComponent(
					currentBufferedImage.getScaledInstance(
						((int)(currentBufferedImage.getWidth()*val/100)),
						((int)(currentBufferedImage.getHeight()*val/100)),
						Image.SCALE_SMOOTH));
			tmp = ictemp.getBufferedImage();
		} else if (command.equals("̤jp")) {
			JTextField textField1 = new JTextField(5);
			JTextField textField2 = new JTextField(5);
			Object[] array = {"пJվ᪺jpG", textField1, "x", textField2};
			int result = JOptionPane.showConfirmDialog(this, array,
						"Y", JOptionPane.OK_CANCEL_OPTION,
						JOptionPane.PLAIN_MESSAGE);
			if (result == JOptionPane.CANCEL_OPTION) {
				return;
			}
			if (textField1.getText() == null || textField2.getText() == null) {
				return;
			}
			ImageComponent ictemp = new ImageComponent(
					currentBufferedImage.getScaledInstance(
						Integer.parseInt(textField1.getText()),
						Integer.parseInt(textField2.getText()),
						Image.SCALE_SMOOTH));
			tmp = ictemp.getBufferedImage();
		} else if (command.equals("Ϭ")) {
			tmp = imageProcess.invert(currentBufferedImage);
		} else if (command.equals("m")) {
			String result = (String)JOptionPane.showInputDialog(this,
					"пJվ᪺m(%)(100Ӫm)G",
					"վm", JOptionPane.OK_CANCEL_OPTION, null, null, null);
			if (result == null) {
				return;
			}
			tmp = imageProcess.hue(currentBufferedImage,
					Integer.parseInt(result));
		} else if (command.equals("ⶥ")) {
			String result = (String)JOptionPane.showInputDialog(this,
					"пJܫ᪺ⶥ(0~255, `255)G",
					"վⶥ", JOptionPane.OK_CANCEL_OPTION, null, null, null);
			if (result == null) {
				return;
			}
			tmp = imageProcess.colorLevel(currentBufferedImage,
					Integer.parseInt(result));
		} else if (command.equals("z")) {
			String result = (String)JOptionPane.showInputDialog(this,
					"пJQnz(%)(100Ӫz)G",
					"վz", JOptionPane.OK_CANCEL_OPTION, null, null, null);
			if (result == null) {
				return;
			}
			tmp = imageProcess.transparency(currentBufferedImage,
					Integer.parseInt(result));
		} else if (command.equals("")) {
			JTextField textField1 = new JTextField(5);
			JTextField textField2 = new JTextField(5);
			JTextField textField3 = new JTextField(5);
			Object[] array = {"пJU⪺ܶq", "Rq(0~255)G",
				textField1, "Gq(0~255)G", textField2, "Bq(0~255)G",
				textField3};
			int result = JOptionPane.showConfirmDialog(this, array,
						"", JOptionPane.OK_CANCEL_OPTION,
						JOptionPane.PLAIN_MESSAGE);
			if (result == JOptionPane.CANCEL_OPTION) {
				return;
			}
			if (textField1.getText() == null ||
					textField2.getText() == null ||
					textField3.getText() == null) {
				return;
			}
			tmp = imageProcess.colorPhase(currentBufferedImage,
					Integer.parseInt(textField1.getText()),
					Integer.parseInt(textField2.getText()),
					Integer.parseInt(textField3.getText()));
		} else if (command.equals("g")) {
			tmp = imageProcess.hmir(currentBufferedImage);
		} else if (command.equals("g")) {
			tmp = imageProcess.vmir(currentBufferedImage);
		} else if (command.equals("180")) {
			tmp = imageProcess.turn180(currentBufferedImage);
		} else if (command.equals("k90")) {
			tmp = imageProcess.turn90(currentBufferedImage);
		} else if (command.equals("90")) {
			tmp = imageProcess.turn90o(currentBufferedImage);
		} else if (command.equals("N")) {
			String result = (String)JOptionPane.showInputDialog(this,
					"пJ઺(X)G",
					"N", JOptionPane.OK_CANCEL_OPTION, null, null, null);
			if (result == null) {
				return;
			}
			tmp = imageProcess.turn(currentBufferedImage,
					Integer.parseInt(result));
		} else if (command.equals("ɧJ")) {
			JTextField textField1 = new JTextField(5);
			JTextField textField2 = new JTextField(5);
			Object[] array = {"пJվ᪺ѪRסG", textField1, "x", textField2};
			int result = JOptionPane.showConfirmDialog(this, array,
						"ɧJ", JOptionPane.OK_CANCEL_OPTION,
						JOptionPane.PLAIN_MESSAGE);
			if (result == JOptionPane.CANCEL_OPTION) {
				return;
			}
			if (textField1.getText() == null || textField2.getText() == null) {
				return;
			}
			tmp = imageProcess.resolution(currentBufferedImage,
					Integer.parseInt(textField1.getText()),
					Integer.parseInt(textField2.getText()));
		} else if (command.equals("W[I")) {
			tmp = imageProcess.pepper(currentBufferedImage);
		} else if (command.equals("ҽk")) {
			tmp = imageProcess.mean(currentBufferedImage);
		} else if (command.equals("hI")) {
			tmp = imageProcess.median(currentBufferedImage);
		} else if (command.equals("BJ")) {
			tmp = imageProcess.pixelDifferent(currentBufferedImage);
		} else if (command.equals("")) {
			tmp = imageProcess.uniform(currentBufferedImage);
		} else if (command.equals("")) {
			return;
		} else {
			JOptionPane.showMessageDialog(this, "?");
			return;
		}
		int count = imageList.size() - 1 - listIndex;
		for (int i = 0; i < count; ++i) {
			imageList.removeLast();
		}
		listIndex = imageList.size() - 1;
		imageList.add(tmp);
		listIndex++;
		imageComponent.setBufferedImage(tmp);
		//set width, height
		Dimension d = imageComponent.getPreferredSize();
		int dx = (int)d.getWidth();
		int dy = (int)d.getHeight();
		//set title
		setTitle(currentFile.getName() + " " + dx + "X" + dy);
		repaint();
		System.gc();
	}

	private void openFile() {
		//cut and modified from Java toturial FileChooserDemo2.java

		//Set up the file chooser.
		if (fcRead == null) {
			fcRead = new JFileChooser();

			//Add a custom file filter and disable the default
			//(Accept All) file filter.
			fcRead.addChoosableFileFilter(new ImageReadFileFilter());
			fcRead.setAcceptAllFileFilterUsed(false);

			//Add the preview pane.
			fcRead.setAccessory(new ImagePreview(fcRead));
		}

		//Show it.
		int returnVal = fcRead.showDialog(this, "}");

		//Process the results.
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			//do the same things as constructor
			currentFile = fcRead.getSelectedFile();
			imageList = new LinkedList();
			ImageComponent ic = new ImageComponent(
					Toolkit.getDefaultToolkit().getImage(currentFile.toString()));
			imageList.add(ic.getBufferedImage());
			listIndex = 0;
			Dimension d = ic.getPreferredSize();
			setTitle(currentFile.getName() + " " + (int)d.getWidth() + "X" + (int)d.getHeight());
			getContentPane().removeAll();
			getContentPane().add(new JScrollPane(ic));
			//]̤jùjp
			pack();
			int dx = this.getWidth();
			int dy = this.getHeight();
			Dimension sd = Toolkit.getDefaultToolkit().getScreenSize();
			int maxdx = (int)sd.getWidth();
			int maxdy = (int)sd.getHeight();
			//dx = dx + 11;
			//dy = dy + 55;
			if (dx > maxdx) {
				dx = maxdx;
			}
			if (dy > maxdy) {
				dy = maxdy;
			}
			setSize(dx, dy);
			setLocation((maxdx - dx) / 2, (maxdy - dy) / 2);
			pack();
		}

		//Reset the file chooser for the next time it's shown.
		fcRead.setSelectedFile(null);
	}

	private void saveFile() {
		/*
		 * Get the extension of a file.
		 */
		String ext = null;
		String s = currentFile.getName();
		int i = s.lastIndexOf('.');
		if (i > 0 &&  i < s.length() - 1) {
			ext = s.substring(i+1).toLowerCase();
		}

		if ((ext == null) || ext.equals("gif") || ext.equals("GIF")) {
			ext = "png";
			String filename = currentFile.getName();
			String filepath = currentFile.getParent();

			try {
				ImageIO.write(((BufferedImage)imageList.get(listIndex)), ext, new File(filepath + "\\" + filename + "." + ext));
			} catch (IOException ioe) {
				ioe.printStackTrace();
			}
			//open file
			currentFile = new File(filepath + "\\" + filename + "." + ext);
			imageList = new LinkedList();
			ImageComponent ic = new ImageComponent(Toolkit.getDefaultToolkit().getImage(
						currentFile.toString()));
			imageList.add(ic.getBufferedImage());
			listIndex = 0;
			Dimension d = ic.getPreferredSize();
			setTitle(currentFile.getName() + " " + (int)d.getWidth() + "X" + (int)d.getHeight());
			getContentPane().removeAll();
			getContentPane().add(new JScrollPane(ic));
			//]̤jùjp
			pack();
			int dx = this.getWidth();
			int dy = this.getHeight();
			Dimension sd = Toolkit.getDefaultToolkit().getScreenSize();
			int maxdx = (int)sd.getWidth();
			int maxdy = (int)sd.getHeight();
			//dx = dx + 11;
			//dy = dy + 55;
			if (dx > maxdx) {
				dx = maxdx;
			}
			if (dy > maxdy) {
				dy = maxdy;
			}
			setSize(dx, dy);
			setLocation((maxdx - dx) / 2, (maxdy - dy) / 2);
			pack();
		} else {

			try {
				ImageIO.write(((BufferedImage)imageList.get(listIndex)), ext, currentFile);
			} catch (IOException ioe) {
				ioe.printStackTrace();
			}
		}
	}		

	private void saveAsFile() {
		//cut and modified from Java toturial FileChooserDemo2.java

		//Set up the file chooser.
		if (fcWrite == null) {
			fcWrite = new JFileChooser();

			//Add a custom file filter and disable the default
			//(Accept All) file filter.
			fcWrite.addChoosableFileFilter(new ImageWriteFileFilter());
			fcWrite.setAcceptAllFileFilterUsed(false);

			//Add the preview pane.
			fcWrite.setAccessory(new ImagePreview(fcWrite));
		}

		//Show it.
		int returnVal = fcWrite.showDialog(this, "sɦ");

		//Process the results.
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			//do the same things as constructor
			currentFile = fcWrite.getSelectedFile();
			//save photo
			String filepath = currentFile.getParent();
			String ext = null;
			String filename = null;
			String name = currentFile.getName();
			int i = name.lastIndexOf('.');

			String[] imgnames = ImageIO.getWriterFormatNames();
			String format = imgnames[0];
			for (int j = 1; j < imgnames.length; ++j) {
				format += "|" + imgnames[j];
			}

			if (name.matches(".*(" + format + ")$")) {
				ext = name.substring(i+1);
				filename = name.substring(0, i);
			} else {
				ext = "png";
				filename = name;
			}

			try {
				ImageIO.write(((BufferedImage)imageList.get(listIndex)), ext, new File(filepath + "\\" + filename + "." + ext));
			} catch (IOException ioe) {
				ioe.printStackTrace();
			}
			//open it
			currentFile = new File(filepath + "\\" + filename + "." + ext);
			imageList = new LinkedList();
			ImageComponent ic = new ImageComponent(Toolkit.getDefaultToolkit().getImage(currentFile.toString()));
			imageList.add(ic.getBufferedImage());
			listIndex = 0;
			Dimension d = ic.getPreferredSize();
			setTitle(currentFile.getName() + " " + (int)d.getWidth() + "X" + (int)d.getHeight());
			getContentPane().removeAll();
			getContentPane().add(new JScrollPane(ic));
			//]̤jùjp
			pack();
			int dx = this.getWidth();
			int dy = this.getHeight();
			Dimension sd = Toolkit.getDefaultToolkit().getScreenSize();
			int maxdx = (int)sd.getWidth();
			int maxdy = (int)sd.getHeight();
			//dx = dx + 11;
			//dy = dy + 55;
			if (dx > maxdx) {
				dx = maxdx;
			}
			if (dy > maxdy) {
				dy = maxdy;
			}
			setSize(dx, dy);
			setLocation((maxdx - dx) / 2, (maxdy - dy) / 2);
			pack();
		}
		fcWrite.setSelectedFile(null);
	}

	private void pageUp() {
		File[] tmpFileList = dirFile.listFiles(new ImageFilenameFilter());
		if (fileIndex == 0) {
			fileIndex = tmpFileList.length - 1;
		} else {
			fileIndex--;
		}
		currentFile = tmpFileList[fileIndex];

		//initialize list
		imageList = new LinkedList();
		//read image from file
		Image image = Toolkit.getDefaultToolkit().getImage(currentFile.toString());
		ImageComponent ictmp = new ImageComponent(image);
		imageComponent.setBufferedImage(ictmp.getBufferedImage());
		//add ImageComponent to list
		imageList.add(imageComponent.getBufferedImage());
		listIndex = 0;
		//set width, height
		Dimension d = imageComponent.getPreferredSize();
		int dx = (int)d.getWidth();
		int dy = (int)d.getHeight();
		//set title
		setTitle(currentFile.getName() + " " + dx + "X" + dy);
		//]̤jùjp
		Dimension sd = Toolkit.getDefaultToolkit().getScreenSize();
		int maxdx = (int)sd.getWidth();
		int maxdy = (int)sd.getHeight();
		//dx = dx + 11;
		//dy = dy + 55;
		pack();
		dx = this.getWidth();
		dy = this.getHeight();
		if (dx > maxdx) {
			dx = maxdx;
		}
		if (dy > maxdy) {
			dy = maxdy;
		}
		setSize(dx, dy);
		setLocation((maxdx - dx) / 2, (maxdy - dy) / 2);
	}

	private void pageDown() {
		File[] tmpFileList = dirFile.listFiles(new ImageFilenameFilter());
		if (fileIndex == tmpFileList.length - 1) {
			fileIndex = 0;
		} else {
			fileIndex++;
		}
		currentFile = tmpFileList[fileIndex];

		//initialize list
		imageList = new LinkedList();
		//read image from file
		Image image = Toolkit.getDefaultToolkit().getImage(currentFile.toString());
		ImageComponent ictmp = new ImageComponent(image);
		imageComponent.setBufferedImage(ictmp.getBufferedImage());
		//add ImageComponent to list
		imageList.add(imageComponent.getBufferedImage());
		listIndex = 0;
		//set width, height
		Dimension d = imageComponent.getPreferredSize();
		int dx = (int)d.getWidth();
		int dy = (int)d.getHeight();
		//set title
		setTitle(currentFile.getName() + " " + dx + "X" + dy);
		//]̤jùjp
		Dimension sd = Toolkit.getDefaultToolkit().getScreenSize();
		int maxdx = (int)sd.getWidth();
		int maxdy = (int)sd.getHeight();
		//dx = dx + 11;
		//dy = dy + 55;
		pack();
		dx = this.getWidth();
		dy = this.getHeight();
		if (dx > maxdx) {
			dx = maxdx;
		}
		if (dy > maxdy) {
			dy = maxdy;
		}
		setSize(dx, dy);
		setLocation((maxdx - dx) / 2, (maxdy - dy) / 2);
	}

	private void undo() {
		if (listIndex == 0) {
			return;
		}
		imageComponent.setBufferedImage((BufferedImage)imageList.get(listIndex - 1));
		listIndex--;
		//set width, height
		Dimension d = imageComponent.getPreferredSize();
		int dx = (int)d.getWidth();
		int dy = (int)d.getHeight();
		//set title
		setTitle(currentFile.getName() + " " + dx + "X" + dy);
	}

	private void redo() {
		if (listIndex == (imageList.size() - 1)) {
			return;
		}
		imageComponent.setBufferedImage((BufferedImage)imageList.get(listIndex + 1));
		listIndex++;
		//set width, height
		Dimension d = imageComponent.getPreferredSize();
		int dx = (int)d.getWidth();
		int dy = (int)d.getHeight();
		//set title
		setTitle(currentFile.getName() + " " + dx + "X" + dy);
	}

	public static void main(String[] args) {
		new ImageFrame(new File("i-mad.jpg"));
	}
}
